import { create } from 'zustand'
import { ListMessagesModel } from "@/feature/messenger/models/ListMessagesModel";
import { showMessages, showMessagesTotalService } from '@/feature/messenger/services/chat/ChatService';
import toast from 'react-hot-toast';


type typeFilterSearch = {
    type: string,
    value: string
}
type TypeActionsMessages = {
    setListMessages: (message: ListMessagesModel[]) => void
    setFilter: (filter: typeFilterSearch) => void
    getListMessages: (id_chatbot: string, sesion: string,loading: boolean) => void
    addItemListMessages: (message: ListMessagesModel) => void
    removeItemListMessages: (message: ListMessagesModel) => void
    updateItemListMessages: (message: ListMessagesModel) => void
    setSearchListMessages: (filter: typeFilterSearch) => void
    clearListMessages: () => void,
    consultHistory: (id_chatbot: string, sesion: string) => void
    getFilesCategory: () => void
    changeFocus: () => void
    setLoading: (loading: boolean) => void
    refreshMessage: () => void
}
type TypeMessages = {
    listMessages: ListMessagesModel[],
    defaultListMessages: ListMessagesModel[],
    filter: typeFilterSearch,
    listMessagesHistory: ListMessagesModel[],
    listCategoryFileConversation: {},
    focus: boolean,
    loading: boolean,
    actions: TypeActionsMessages
}

const useMessagesStore = create<TypeMessages>((set, get) => ({
    listMessages: [],
    defaultListMessages: [],
    filter: { type: 'message', value: '' },
    listMessagesHistory: [],
    listCategoryFileConversation: {},
    focus: false,
    loading: false,
    actions: {
        setListMessages: (listMessages: ListMessagesModel[]) => {

            let countOldMessages = get().defaultListMessages.length + get().listMessagesHistory.length;

            set({ defaultListMessages: listMessages })
            const allMessages = get().listMessagesHistory.concat(get().defaultListMessages);
            const filteredConversations = allMessages.filter(item => {
                const nameComplete = `${item.conversacion.toLowerCase()}`;
                return nameComplete.includes(get().filter.value.toLowerCase());
            });
            set({ listMessages: filteredConversations })
            get().actions.getFilesCategory()

            if (countOldMessages != allMessages.length) {
                setTimeout(() => {
                    const element = document.getElementById("chating-messages");
                    if (element !== null) {
                        element.scrollTop = element.scrollHeight * 2;
                    }
                }, 200)
            }
        },
        setFilter: (filter: typeFilterSearch) => set({ filter: filter }),
        getListMessages: (id_chatbot: string, sesion: string,loading: boolean = false) => {
            if (id_chatbot === '') {
                return
            }
            if (loading) {
                get().actions.setLoading(true)
                showMessages(id_chatbot, sesion).then((res) => {
                    if (res.data !== null) {
                        get().actions.setListMessages(res.data)
                    } else {
                        get().actions.setListMessages([])
                    }
                }).finally(() => {
                    get().actions.setLoading(false)
                })
            }
        },
        addItemListMessages: (message: ListMessagesModel) => {
            set((state) => ({ listMessages: [...state.listMessages, message] }))
            set((state) => ({ defaultListMessages: [...state.defaultListMessages, message] }))
            setTimeout(() => {
                const element = document.getElementById("chating-messages");
                if (element !== null) {
                    element.scrollTop = element.scrollHeight * 2;
                }
            }, 200)
        },
        removeItemListMessages: (message: ListMessagesModel) => set((state) => ({ listMessages: state.listMessages.filter((item) => item.id_log !== message.id_log) })),
        updateItemListMessages: (message: ListMessagesModel) => set((state) => ({ listMessages: state.listMessages.map((item) => item.id_log === message.id_log ? message : item) })),
        setSearchListMessages: (filter: typeFilterSearch) => {
            set({ filter: filter })
            if (filter.type === 'message') {
                const allMessages = get().listMessagesHistory.concat(get().defaultListMessages);
                const filteredConversations = allMessages.filter(item => {
                    const nameComplete = `${item.conversacion.toLowerCase()}`;
                    return nameComplete.includes(filter.value.toLowerCase());
                });
                set({ listMessages: filteredConversations })
            }
        },
        clearListMessages: () => set({ listMessages: [], defaultListMessages: [], listMessagesHistory: [] }),
        consultHistory: (id_chatbot: string, sesion: string) => {
            get().actions.setLoading(true)
            showMessagesTotalService(id_chatbot, sesion).then((res) => {
                if (res.data !== null && res.data.length > 0) {
                    set({ listMessagesHistory: res.data })
                    toast.success("Se encontraron historial de conversaciones");
                } else {
                    set({ listMessagesHistory: [] })
                    toast.error("No se encontraron historial de conversaciones");
                }
                get().actions.refreshMessage()
            }).finally(() => {
                get().actions.setLoading(false)
            })
        },
        getFilesCategory: () => {
            const typePhoto: any = {
                '.jpg': true,
                '.png': true,
                '.jpeg': true,
                '.gif': true
            };
            const typeFile: any = {
                '.txt': true,
                '.pdf': true,
                '.docx': true,
                '.pptx': true,
                '.xlsx': true,
                '.zip': true,
                '.rar': true,
            }
            const list = get().listMessagesHistory.concat(get().defaultListMessages);
            const listCategoryPhotos: any = [];
            const listCategoryFiles: any = [];
            const listCategoryLink: any = [];
            list.forEach((item: ListMessagesModel) => {

                if (typePhoto[item.tipomensaje] == true) {
                    listCategoryPhotos.push(item.conversacion);
                } else if (typeFile[item.tipomensaje] == true) {
                    let array = item.conversacion.split('/');
                    const nameFile = array.pop();
                    listCategoryFiles.push({
                        name: nameFile,
                        url: item.conversacion,
                        type: item.tipomensaje.replace('.', ''),
                        date: item.fecha_sistema
                    });
                } else if (item.tipomensaje == 'text') {
                    let texto = item.conversacion;
                    // Expresión regular para buscar una URL
                    const expresionRegular = /(https?:\/\/([^\/]+)\/)/;
                    // Extraer la URL del texto utilizando match()
                    const match = texto.match(expresionRegular);
                    // La URL completa estará en el primer elemento del array resultante
                    const urlCompleta = match ? match['input'] : null;
                    // El nombre de dominio estará en el segundo elemento del array resultante
                    const nombreDominio = match ? match[2] : null;
                    if (match) {
                        listCategoryLink.push({
                            name: nombreDominio,
                            url: urlCompleta,
                        });
                    }
                }
            })
            const data = { listCategoryPhotos, listCategoryFiles, listCategoryLink };
            set({ listCategoryFileConversation: data })
        },
        changeFocus: () => {
            set({ focus: !get().focus })
        },
        setLoading: (loading: boolean) => set({ loading: loading }),
        refreshMessage: () => {
            const allMessages = get().listMessagesHistory.concat(get().defaultListMessages);
            const filteredConversations = allMessages.filter(item => {
                const nameComplete = `${item.conversacion.toLowerCase()}`;
                return nameComplete.includes(get().filter.value.toLowerCase());
            });
            set({ listMessages: filteredConversations })
            get().actions.getFilesCategory()
        },
    }
}))

export default useMessagesStore